/**
 * @function validateDataGiven
 * @returns {?Object} - Error if failed
 */
'use strict'

const apemanerror = require('apemanerror')
const ApErrorCodes = apemanerror.ApErrorCodes

/** @lends validateDataGiven */
function validateDataGiven (data) {
  let missing = !data
  if (missing) {
    return validateDataGiven.missingError()
  }
  return null
}

Object.assign(validateDataGiven, {
  bulk (data) {
    let missing = data.length === 0
    if (missing) {
      return validateDataGiven.missingError()
    }
  },

  missingError () {
    let code = ApErrorCodes.RESOURCE_DATA_MISSING_ERROR
    return Object.assign(
      apemanerror.newError(code, '/data', {
        trigger: 'RestHandler'
      }),
      { status: '400' }
    )
  }
})

module.exports = validateDataGiven
